/* acrostic-answer-list-row.c
 *
 * Copyright 2025 Tanmay Patil <tanmaynpatil105@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#include "acrostic-answer-list-row.h"

enum
{
  CHANGED,
  N_SIGNALS,
};

static guint obj_signals[N_SIGNALS] = { 0 };

struct _AcrosticAnswerListRow
{
  GObject parent_object;

  gchar *answer;
  gboolean selected;
};

static void  acrostic_answer_list_row_init       (AcrosticAnswerListRow      *self);
static void  acrostic_answer_list_row_class_init (AcrosticAnswerListRowClass *klass);
static void  acrostic_answer_list_row_dispose    (GObject                    *object);

G_DEFINE_TYPE (AcrosticAnswerListRow, acrostic_answer_list_row, G_TYPE_OBJECT);

static void
acrostic_answer_list_row_init (AcrosticAnswerListRow *self)
{

}

static void
acrostic_answer_list_row_class_init (AcrosticAnswerListRowClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);

  object_class->dispose = acrostic_answer_list_row_dispose;

  obj_signals[CHANGED] =
    g_signal_new ("changed",
                  ACROSTIC_TYPE_ANSWER_LIST_ROW,
                  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                  0,
                  NULL, NULL,
                  NULL,
                  G_TYPE_NONE, 0);
}

static void
acrostic_answer_list_row_dispose (GObject *object)
{
  AcrosticAnswerListRow *self;

  self = ACROSTIC_ANSWER_LIST_ROW (object);

  g_clear_pointer (&self->answer, g_free);

  G_OBJECT_CLASS (acrostic_answer_list_row_parent_class)->dispose (object);
}

const gchar *
acrostic_answer_list_row_get_answer (AcrosticAnswerListRow *self)
{
  g_return_val_if_fail (ACROSTIC_IS_ANSWER_LIST_ROW (self), NULL);

  return self->answer;
}

gboolean
acrostic_answer_list_row_get_selected (AcrosticAnswerListRow *self)
{
  g_return_val_if_fail (ACROSTIC_IS_ANSWER_LIST_ROW (self), FALSE);

  return self->selected;
}

AcrosticAnswerListRow *
acrostic_answer_list_row_new (const gchar *answer,
                              gboolean     selected)
{
  AcrosticAnswerListRow *row;

  row = g_object_new (ACROSTIC_TYPE_ANSWER_LIST_ROW, NULL);
  row->answer = g_strdup (answer);
  row->selected = selected;

  return row;
}

void
acrostic_answer_list_set_answer (AcrosticAnswerListRow *self,
                                 const gchar           *answer)
{
  g_return_if_fail (ACROSTIC_IS_ANSWER_LIST_ROW (self));

  g_clear_pointer (&self->answer, g_free);

  self->answer = g_strdup (answer);

  g_signal_emit (self, obj_signals [CHANGED], 0);
}

void
acrostic_answer_list_row_set_selected (AcrosticAnswerListRow *self,
                                       gboolean               selected)
{
  g_return_if_fail (ACROSTIC_IS_ANSWER_LIST_ROW (self));

  self->selected = selected;

  g_signal_emit (self, obj_signals [CHANGED], 0);
}
