/* crosswords-app-startup.c
 *
 * Copyright 2025 Mahmoud Abdelghany <mahmoud.a.rageh@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */


#include "crosswords-config.h"
#include <gio/gio.h>
#include <glib/gi18n-lib.h>
#include "crosswords-init-startup.h"
#include "puzzle-downloader.h"
#include "puzzle-set-list.h"
#include "puzzle-set.h"


static GSettings *global_settings = NULL;


static void
startup_auto_download (void)
{
  g_autoptr (GError) error = NULL;
  gboolean auto_download;
  GNetworkMonitor *monitor;
  monitor = g_network_monitor_get_default ();

  global_settings = g_settings_new ("org.gnome.Crosswords");
  auto_download = g_settings_get_boolean (global_settings, "auto-download-puzzle-sets");

  if (auto_download)
    {
      GStrv shown_puzzle_sets;
      GListModel *puzzle_set_list;
      guint n_items;

      if (!g_network_monitor_get_network_available (monitor))
        return;

      shown_puzzle_sets = g_settings_get_strv (global_settings, "shown-puzzle-sets");
      puzzle_set_list = puzzle_set_list_get ();
      n_items = g_list_model_get_n_items (puzzle_set_list);

      for (guint i = 0; i < n_items; i++)
        {
          g_autoptr (PuzzleSet) puzzle_set = NULL;
          const gchar *id;
          gboolean is_enabled = FALSE;

          puzzle_set = g_list_model_get_item (puzzle_set_list, i);
          id = puzzle_set_get_id (puzzle_set);

          for (gint j = 0; shown_puzzle_sets[j] != NULL; j++)
            {
              if (g_strcmp0 (id, shown_puzzle_sets[j]) == 0)
                {
                  is_enabled = TRUE;
                  break;
                }
            }

          if (is_enabled && puzzle_set_get_auto_download (puzzle_set))
            {
              PuzzleDownloader *downloader;
              GCancellable *downloader_cancellable;

              downloader = puzzle_set_get_downloader(puzzle_set);
              downloader_cancellable = puzzle_set_get_downloader_cancellable(puzzle_set);

              if (downloader == NULL)
                  continue;

              puzzle_downloader_run_auto (downloader, downloader_cancellable, &error);
              puzzle_set_set_downloader_state (puzzle_set,
                                               DOWNLOADER_STATE_DOWNLOADING);
            }
        }
    }
}

void
crosswords_init_startup (void)
{
  startup_auto_download ();
}
